@@ -7,6 +7,7 @@ from loguru import logger
 from .trainer import generate_dataloader
 from .onnx import export_onnx, inference_from_onnx
 from .ner_utils import LabeledText, show_ner_datainfo, get_ner_preds_reviews, save_ner_preds, load_ner_examples, load_ner_labeled_examples
+from .ner_utils import to_train_poplar, to_reviews_poplar
 from .glue_utils import show_glue_datainfo, load_glue_examples, save_glue_preds
 from .common_args import add_common_args
 
@@ -53,6 +54,7 @@ class CommonParams(Params):
     dataset_name: str = None
     experiment_name: str = None
     tracking_uri: str = None
+    artifact_path: str = None
     train_file: str = None
     eval_file: str = None
     test_file: str = None
@@ -69,17 +71,41 @@ class CommonParams(Params):
     fold: int = 0
     num_augements: int = 0
     enable_kd: bool = False
+    kd_coeff: float = 1.0
+    kd_decay: float = 0.995
+    enable_sda: bool = False
+    sda_teachers: int = 2
+    sda_coeff: float = 1.0
+    sda_decay: float = 0.995
     loss_type: str = "CrossEntropyLoss"
+    focalloss_gamma: float = 1.5
+    focalloss_alpha: List = None
     model_type: str = "bert"
     model_path: str = None
     train_rate: float = 0.9
     fp16: bool = True
+    seed: int = 8864
+    best_index: str = "f1"
+    random_type: str = None
+
+    def __post_init__(self):
+        tracking_uri = self.tracking_uri
+        if 'TRACKING_URI' in os.environ:
+            tracking_uri = os.environ['TRACKING_URI']
+        self.tracking_uri = tracking_uri
+
+        artifact_path = self.artifact_path
+        if 'ARTIFACT_PATH' in os.environ:
+            artifact_path = os.environ['ARTIFACT_PATH']
+        self.artifact_path = artifact_path
 
 
 @dataclass
 class NerParams(Params):
     ner_labels: List[str] = field(default_factory=list)
     ner_type: str = "crf"
+    no_crf_loss: bool = False
+    soft_label: bool = False
 
 
 @dataclass
@@ -114,7 +140,8 @@ def log_global_params(args, experiment_params):
 
 def archive_local_model(args, submission_file):
     if args.do_experiment:
-        mlflow.log_param("submission_file", submission_file)
+        mlflow.log_param(f"{args.dataset_name}_submission_file",
+                         submission_file)
         mlflow.log_artifact(submission_file)
         logger.info(f"Log {submission_file} to tracking.mlflow.")
 
@@ -126,6 +153,11 @@ def archive_local_model(args, submission_file):
         f"Archive local model({args.local_id}) {args.latest_dir} to {args.local_dir}"
     )
 
+    #  os.remove(args.local_id_file)
+    #  logger.info(
+    #      f"Unlink {args.local_id_file} after archived local model({args.local_id})."
+    #  )
+
 
 def augement_entities(all_text_entities, labels_map):
     aug_tokens = []
