@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import random, copy, json
+import os, random, copy, json
 from tqdm import tqdm
 from loguru import logger
 from theta.utils import seg_generator
@@ -170,7 +170,8 @@ def save_ner_preds(args, preds, test_examples):
                                                        args.seg_len,
                                                        args.seg_backoff)
 
-    reviews_file = f"{args.latest_dir}/{args.dataset_name}_reviews_fold{args.fold}.json"
+    reviews_file = f"{args.latest_dir}/{args.dataset_name}_reviews_{args.local_id}.json"
+
     json.dump(reviews, open(reviews_file, 'w'), ensure_ascii=False, indent=2)
     logger.info(f"Reviews file: {reviews_file}")
 
@@ -188,10 +189,11 @@ def save_ner_preds(args, preds, test_examples):
 
     #  ----- Tracking -----
     if args.do_experiment:
-        mlflow.log_param("reviews_file", reviews_file)
-        mlflow.log_artifact(reviews_file)
-        mlflow.log_param("category_mentions_file", category_mentions_file)
-        mlflow.log_artifact(category_mentions_file)
+        mlflow.log_param(f"{args.dataset_name}_reviews_file", reviews_file)
+        mlflow.log_artifact(reviews_file, args.artifact_path)
+        mlflow.log_param(f"{args.dataset_name}_category_mentions_file",
+                         category_mentions_file)
+        mlflow.log_artifact(category_mentions_file, args.artifact_path)
 
     return reviews_file, category_mentions_file
 
@@ -264,9 +266,11 @@ def data_seg_generator(lines,
                        seg_backoff=0,
                        num_augements=0,
                        allow_overlap=False):
+    assert seg_backoff >= 0 and seg_backoff <= int(seg_len / 2)
     all_text_entities = []
     labels_map = {}
 
+    num_overlap = 0
     for i, s in enumerate(tqdm(lines)):
         #  logger.debug(f"s: {s}")
         guid = str(i)
@@ -292,19 +296,21 @@ def data_seg_generator(lines,
                 if e >= us[0] and e <= us[1]:
                     overlap = True
                     break
-            if overlap and not allow_overlap:
-                logger.warning(
-                    f"Overlap! {i} mention: {entity.mention}({s}:{e}), used_span: {used_span}"
-                )
-                continue
+            if overlap:
+                num_overlap += 1
+                if not allow_overlap:
+                    #  logger.warning(
+                    #      f"Overlap! {i} mention: {entity.mention}({s}:{e}), used_span: {used_span}"
+                    #  )
+                    continue
             used_span.append((s, e))
 
             new_entities.append(entity)
         entities = new_entities
 
         seg_offset = 0
-        if seg_len <= 0:
-            seg_len = max_seq_length
+        #  if seg_len <= 0:
+        #      seg_len = max_seq_length
 
         for (seg_text, ), text_offset in seg_generator((text, ), seg_len,
                                                        seg_backoff):
@@ -358,6 +364,8 @@ def data_seg_generator(lines,
 
             seg_offset += seg_len - seg_backoff
 
+    logger.warning(f"num_overlap: {num_overlap}")
+
     if num_augements > 0:
         aug_tokens = augement_entities(all_text_entities,
                                        labels_map,
@@ -480,3 +488,163 @@ def show_ner_datainfo(ner_labels, train_data_generator, train_file,
     logger.info(f"min: {np.min(test_lengths)}")
     logger.info(f"max: {np.max(test_lengths)}")
     logger.info(f"")
+
+
+def to_train_poplar(args,
+                    train_data_generator,
+                    ner_labels,
+                    start_page=0,
+                    max_pages=100):
+    poplar_json = {
+        "content": "",
+        "labelCategories": [],
+        "labels": [],
+        "connectionCategories": [],
+        "connections": []
+    }
+
+    poplar_colorset = [
+        '#007bff', '#17a2b8', '#28a745', '#fd7e14', '#e83e8c', '#dc3545',
+        '#20c997', '#ffc107', '#007bff'
+    ]
+    label2id = {x: i for i, x in enumerate(ner_labels)}
+    label_categories = poplar_json['labelCategories']
+    for _id, x in enumerate(ner_labels):
+        label_categories.append({
+            "id":
+            _id,
+            "text":
+            x,
+            "color":
+            poplar_colorset[label2id[x] % len(poplar_colorset)],
+            "borderColor":
+            "#cccccc"
+        })
+
+    poplar_labels = poplar_json['labels']
+    poplar_content = ""
+    eid = 0
+    num_pages = 0
+    page_offset = 0
+    for guid, text, _, entities in train_data_generator(args.train_file):
+        if num_pages < start_page:
+            num_pages += 1
+            continue
+
+        page_head = f"\n-------------------- {guid} Begin --------------------\n\n"
+        page_tail = f"\n-------------------- {guid} End --------------------\n\n"
+        poplar_content += page_head + f"{text}" + page_tail
+
+        for entity in entities:
+            poplar_labels.append({
+                "id":
+                eid,
+                "categoryId":
+                label2id[entity.category],
+                "startIndex":
+                page_offset + len(page_head) + entity.start,
+                "endIndex":
+                page_offset + len(page_head) + entity.end + 1,
+            })
+            eid += 1
+
+        num_pages += 1
+        if num_pages - start_page >= max_pages:
+            break
+
+        page_offset = len(poplar_content)
+
+    poplar_json["content"] = poplar_content
+    poplar_json['labels'] = poplar_labels
+
+    if not os.path.exists("./poplar"):
+        os.makedirs("./poplar")
+    poplar_data_file = f"./poplar/poplar_train_data_{args.local_id}_{max_pages}_{start_page}.json"
+    json.dump(poplar_json,
+              open(poplar_data_file, 'w'),
+              ensure_ascii=False,
+              indent=2)
+    logger.info(f"Saved {poplar_data_file}")
+
+
+def to_reviews_poplar(args, ner_labels, start_page=0, max_pages=100):
+    poplar_json = {
+        "content": "",
+        "labelCategories": [],
+        "labels": [],
+        "connectionCategories": [],
+        "connections": []
+    }
+
+    poplar_colorset = [
+        '#007bff', '#17a2b8', '#28a745', '#fd7e14', '#e83e8c', '#dc3545',
+        '#20c997', '#ffc107', '#007bff'
+    ]
+    label2id = {x: i for i, x in enumerate(ner_labels)}
+    label_categories = poplar_json['labelCategories']
+    for _id, x in enumerate(ner_labels):
+        label_categories.append({
+            "id":
+            _id,
+            "text":
+            x,
+            "color":
+            poplar_colorset[label2id[x] % len(poplar_colorset)],
+            "borderColor":
+            "#cccccc"
+        })
+
+    poplar_labels = poplar_json['labels']
+    poplar_content = ""
+    num_pages = 0
+    page_offset = 0
+
+    reviews_file = f"{args.latest_dir}/{args.dataset_name}_reviews_{args.local_id}.json"
+    reviews = json.load(open(reviews_file, 'r'))
+
+    eid = 0
+    for guid, json_data in reviews.items():
+        if num_pages < start_page:
+            num_pages += 1
+            continue
+        text = json_data['text']
+
+        page_head = f"\n-------------------- {guid} Begin --------------------\n\n"
+        page_tail = f"\n-------------------- {guid} End --------------------\n\n"
+        poplar_content += page_head + f"{text}" + page_tail
+
+        for json_entity in json_data['entities']:
+            category = json_entity['category']
+            mention = json_entity['mention']
+            start_char = json_entity['start']
+            end_char = json_entity['end']
+
+            poplar_labels.append({
+                "id":
+                eid,
+                "categoryId":
+                label2id[category],
+                "startIndex":
+                page_offset + len(page_head) + start_char,
+                "endIndex":
+                page_offset + len(page_head) + end_char + 1,
+            })
+            eid += 1
+
+        num_pages += 1
+        if num_pages - start_page >= max_pages:
+            break
+
+        page_offset = len(poplar_content)
+
+    poplar_json["content"] = poplar_content
+    poplar_json['labels'] = poplar_labels
+
+    if not os.path.exists("./poplar"):
+        os.makedirs("./poplar")
+    poplar_data_file = f"./poplar/poplar_reviews_data_{args.local_id}_{max_pages}_{start_page}.json"
+    json.dump(poplar_json,
+              open(poplar_data_file, 'w'),
+              ensure_ascii=False,
+              indent=2)
+    logger.info(f"Saved {poplar_data_file}")
